# -*- coding: utf-8 -*-

from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneWithPackageLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2

import imio.history
import unittest


class PloneWithHistoryLayer(PloneWithPackageLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""
        # Load ZCML
        self.loadZCML(
            package=imio.history,
            name='testing.zcml'
        )
        z2.installProduct(app, 'imio.history')

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        # Install into Plone site using portal_setup
        applyProfile(portal, 'imio.history:testing')

        # Login and create some test content
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        folder_id = portal.invokeFactory('Folder', 'folder')
        portal[folder_id].reindexObject()

        # make sure we have a default workflow
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')

        # Commit so that the test browser sees these objects
        import transaction
        transaction.commit()

    def tearDownZope(self, app):
        """Tear down Zope."""
        z2.uninstallProduct(app, 'imio.history')


FIXTURE = PloneWithHistoryLayer(
    name="FIXTURE")


INTEGRATION = IntegrationTesting(
    bases=(FIXTURE,),
    name="INTEGRATION")


FUNCTIONAL = FunctionalTesting(
    bases=(FIXTURE,),
    name="FUNCTIONAL")


ACCEPTANCE = FunctionalTesting(
    bases=(
        FIXTURE,
        AUTOLOGIN_LIBRARY_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name="ACCEPTANCE")


class IntegrationTestCase(unittest.TestCase):
    """Base class for integration tests."""

    layer = INTEGRATION

    def setUp(self):
        super(IntegrationTestCase, self).setUp()
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.catalog = self.portal.portal_catalog
        self.wft = self.portal.portal_workflow



# -*- coding: utf-8 -*-

from imio.history.config import HISTORY_COMMENT_NOT_VIEWABLE
from imio.history.interfaces import IImioHistory
from imio.history.testing import IntegrationTestCase
from plone import api
from plone.memoize.instance import Memojito
from Products.Five.browser import BrowserView
from zope.component import getAdapter
from zope.component import getMultiAdapter
from zope.viewlet.interfaces import IViewletManager


class TestDocumentByLineViewlet(IntegrationTestCase):

    def setUp(self):
        super(TestDocumentByLineViewlet, self).setUp()
        # get the viewlet
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        view = BrowserView(doc, self.portal.REQUEST)
        manager = getMultiAdapter(
            (doc, self.portal.REQUEST, view),
            IViewletManager,
            'plone.belowcontenttitle')
        manager.update()
        self.viewlet = manager.get(u'imio.history.documentbyline')

    def test_show_history(self):
        """Test the show_history method.
           The history is shown in every case except if 'ajax_load' is found in the REQUEST."""
        self.assertTrue(self.viewlet.show_history())
        # show_history is False if displayed in a popup, aka 'ajax_load' in the REQUEST
        self.portal.REQUEST.set('ajax_load', True)
        self.assertFalse(self.viewlet.show_history())

    def test_highlight_history_link(self):
        """Test the highlight_history_link method.
           History link will be highlighted if last event had a comment and
           if that comment is not an ignorable comment."""
        adapter = getAdapter(self.portal.doc, IImioHistory, 'workflow')
        # not highlighted because '' is an ignored comment
        history = adapter.getHistory()
        self.assertFalse(history[-1]['comments'])
        self.assertFalse(self.viewlet.highlight_history_link())

        # now 'publish' the doc and add a comment, last event has a comment
        self.wft.doActionFor(self.portal.doc, 'publish', comment='my publish comment')
        # clean memoize
        getattr(adapter, Memojito.propname).clear()
        history = adapter.getHistory()
        self.assertTrue(self.viewlet.highlight_history_link())
        self.assertFalse(history[-1]['comments'] in adapter.ignorableHistoryComments())

        # now test that the 'you can not access this comment' is an ignored message
        self.wft.doActionFor(self.portal.doc, 'retract', comment=HISTORY_COMMENT_NOT_VIEWABLE)
        getattr(adapter, Memojito.propname).clear()
        history = adapter.getHistory()
        self.assertFalse(self.viewlet.highlight_history_link())
        self.assertTrue(history[-1]['comments'] in adapter.ignorableHistoryComments())

        # test that it works if no history
        # it is the case if we changed used workflow
        self.wft.setChainForPortalTypes(('Document', ), ('intranet_workflow',))
        getattr(adapter, Memojito.propname).clear()
        history = adapter.getHistory()
        self.assertFalse(self.viewlet.highlight_history_link())
        self.assertTrue(history == [])



# -*- coding: utf-8 -*-

from imio.history.adapters import ImioRevisionHistoryAdapter
from imio.history.adapters import ImioWfHistoryAdapter


class TestingImioWfHistoryAdapter(ImioWfHistoryAdapter):

    def mayViewEvent(self, event):
        """See docstring in interfaces.py."""
        if self.request.get('hide_wf_history_event', False) and \
           event['action'] == 'publish':
            return False
        return True

    def mayViewComment(self, event):
        """See docstring in interfaces.py."""
        if self.request.get('hide_wf_history_comment', False):
            return False
        return True


class TestingImioRevisionHistoryAdapter(ImioRevisionHistoryAdapter):

    def mayViewEvent(self, event):
        """See docstring in interfaces.py."""
        if self.request.get('hide_revisions_event', False):
            return False
        return True

    def mayViewComment(self, event):
        """See docstring in interfaces.py."""
        if self.request.get('hide_revisions_comment', False):
            return False
        return True



# -*- coding: utf-8 -*-

from imio.history.testing import IntegrationTestCase
from plone import api


class TestVersionPreview(IntegrationTestCase):

    """Test IHVersionPreviewView, it is there to be registered for relevant content types."""

    def test_view(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            title="Document title 1",
            container=self.portal)
        # make 2 new versions of doc and change title each time
        pr = self.portal.portal_repository
        pr.save(obj=doc)
        doc.setTitle("Document title 2")
        doc.reindexObject(idxs=['Title', ])
        pr.save(obj=doc)
        doc.setTitle("Document title 3")
        doc.reindexObject(idxs=['Title', ])
        # the view __call__ receives a version_id
        # it will store in self.versioned_object the version_id object
        # so it is available in the template
        view = doc.restrictedTraverse('@@history-version-preview')
        # current
        self.assertEquals(view.context.Title(),
                          "Document title 3")
        # version_id 1
        view(version_id=1)
        self.assertEquals(view.versioned_object.Title(),
                          "Document title 1")
        # version_id 2
        view(version_id=2)
        self.assertEquals(view.versioned_object.Title(),
                          "Document title 2")



# value displayed in the object history table if a comment is not viewable
HISTORY_COMMENT_NOT_VIEWABLE = u"<span class='discreet'>Access to this comment is restricted.</span>"

# value displayed in the object history table if a revision is not viewable
HISTORY_REVISION_NOT_VIEWABLE = u"<span class='discreet'>Access to this revision is restricted.</span>"

# comments that will be ignored regarding the history link highlight when comment in last event
DEFAULT_IGNORABLE_COMMENTS = (HISTORY_COMMENT_NOT_VIEWABLE, )
